"""
Implementing the ephemeral earl:extend property
by Sean B. Palmer, http://purl.org/net/sbp/
"""

import eep, earlapi

EARL_NS = 'http://www.w3.org/2001/03/earl/1.0#'
earl = eep.Namespace(EARL_NS)
rdf = eep.Namespace('http://www.w3.org/1999/02/22-rdf-syntax-ns#')
daml = eep.Namespace('http://www.daml.org/2001/03/daml+oil#')

S, P, O = 0, 1, 2

# Extensions: Write extensions, and put them up on the Web. The 
# type of stuff: validity properties, unique. properties, etc.

class NTriplesExtendParser: 
   def __init__(self, store=None): 
      self.store = []
      if store: 
         for triple in store: self.store.append(triple)

   def feed(store): 
      for triple in store: self.store.append(triple)

   def returnExtensions(self): 
      # return validity properties (and validity), 
      # and unique properties
      ValidityProperties, UniqueProperties = {}, []
      for triple in self.store: 
         x = eep.nquery('?x <%s> <%s> .' % (rdf.type, earl.ValidityProperty), 
                        self.store)
         if len(x) > 0: 
            for t in x: 
               note = t[S].v
               b = eep.nquery('<%s> <%s> ?x .' % (note, earl.validity), 
                                                   self.store)
               if len(b) == 1: ValidityProperties[note] = b[0][2].v
      x = eep.nquery('?x <%s> <%s> .' % (rdf.type, daml.UniqueProperty), 
                                         self.store)
      if len(x) > 0: 
         for t in x: UniqueProperties.append(t[S].v)
      return ValidityProperties, UniqueProperties

class EARLExpatParser: 
   """A parser for canonical EARL 1.0 Extensions, built on Expat."""

   def __init__(self, parser): 
      parser.StartElementHandler = self.startElement
      parser.EndElementHandler = self.endElement
      parser.CharacterDataHandler = self.charData

   def startElement(self, name, attributes): 
      pass

   def endElement(self, name): 
      pass

   def charData(self, data): 
      pass

def extend(uri): 
   # get URI, parse extensions
   pass

def test(): 
   blargh = '<http://infomesh.net/test#kindaValid> <%s> ' % rdf.type
   blargh += '<http://www.w3.org/2001/03/earl/1.0#ValidityProperty> .\n'
   blargh += '<http://infomesh.net/test#kindaValid> <%s> ' % earl.validity
   blargh += '<http://www.w3.org/2001/03/earl/1.0#Pass> .\n'
   blargh += '<http://infomesh.net/test#version> <%s> ' % rdf.type
   blargh += '<http://www.daml.org/2001/03/daml+oil#UniqueProperty> .'

   print NTriplesExtendParser(eep.parse(blargh)).returnExtensions()

if __name__=="__main__": 
   print earl.extend
   test()