#!/usr/bin/python
import sys, re, cgi, urllib, urllib2, md5
import eep, earlapi

RDF_NS = 'http://www.w3.org/1999/02/22-rdf-syntax-ns#'
RDFS_NS = 'http://www.w3.org/2000/01/rdf-schema#'
EARL_NS = 'http://www.w3.org/2001/03/earl/1.0#' # let's use the real NS

rdf = eep.Namespace(RDF_NS) # the RDF namespace
rdfs = eep.Namespace(RDFS_NS) # the RDFS namespace
earl = eep.Namespace(EARL_NS) # the EARL namespace
inf = eep.Namespace('http://infomesh.net/2002/01/md5#')

def stripTags(s): 
   tags = re.compile(r'(<[^!][^>]*>)', re.S).findall(s)
   for tag in tags: s = s.replace(tag, '') # strip tags
   return str(s)

def getTags(s): 
   tags = re.compile(r'(<[^!][^>]*>)', re.S).findall(s)
   return str(''.join(tags))

def getAttributes(s): 
   tags = re.compile(r'<([^!/][^>]*)>', re.S).findall(s)
   attributes = re.compile(r'"([^"]*)"', re.S).findall(''.join(tags))
   return str(''.join(attributes))

def getHeadings(s): 
   headings = re.compile(r'<[hH]\d[^>]*>([^<]*)</[hH]\d>', re.S).findall(s)
   headings = [stripTagsAndComments(heading) for heading in headings]
   return str(''.join(headings))

def stripComments(s): 
   comments = re.compile(r'(<!--.*?[^-]-->)', re.S).findall(s)
   for comment in comments: s = s.replace(comment, '') # strip comments
   return str(s)

def stripTagsAndComments(s): 
   s = stripTags(s)
   return str(stripComments(s))

def stripWhitespace(s): 
   for ws in ' \t\n\r\f\v': s = s.replace(ws, '')
   return str(s)

def stripAll(s): 
   s = stripTagsAndComments(s)
   return str(stripWhitespace(s))

def hash(s): 
   return md5.new(s).hexdigest()

def earlHash(uri): 
   try: data = urllib2.urlopen(uri).read()
   except: return ''
   b = eep.b()
   r = '%s <%s> <%s> .\n'  % (b, earl.reprOf, uri)
   r += '%s <%s> "%s" .\n' % (b, earl.date, earlapi.earlDateTime())
   r += '%s <%s> "%s" .\n' % (b, inf.stripTagsMD5, hash(stripTags(data[:])))
   r += '%s <%s> "%s" .\n' % (b, inf.getTagsMD5, hash(getTags(data[:])))
   r += '%s <%s> "%s" .\n' % (b, inf.getHeadingsMD5, hash(getHeadings(data[:])))
   r += '%s <%s> "%s" .\n' % (b, inf.stripCommentsMD5, 
                                 hash(stripComments(data[:])))
   r += '%s <%s> "%s" .\n' % (b, inf.stripTagsAndCommentsMD5, 
                                 hash(stripTagsAndComments(data[:])))
   r += '%s <%s> "%s" .\n' % (b, inf.stripWhitespaceMD5, 
                                 hash(stripWhitespace(data[:])))
   r += '%s <%s> "%s" .\n' % (b, inf.stripAllMD5, hash(stripAll(data[:])))
   r += '%s <%s> "%s" .\n' % (b, inf.getAttributesMD5, 
                                 hash(getAttributes(data[:])))
   return r

def doHash(uri): 
   try: data = urllib2.urlopen(uri).read()
   except: sys.exit(0)
   print 'uri: '+uri
   print 'MD5 stripTags: '+hash(stripTags(data[:]))
   # print stripTags(data[:])
   print 'MD5 getTags: '+hash(getTags(data[:]))
   # print getTags(data[:])
   print 'MD5 getHeadings: '+hash(getHeadings(data[:]))
   # print getHeadings(data[:])
   print 'MD5 stripComments: '+hash(stripComments(data[:]))
   # print stripComments(data[:])
   print 'MD5 stripTagsAndComments: '+hash(stripTagsAndComments(data[:]))
   # print stripTagsAndComments(data[:])
   print 'MD5 stripWhitespace: '+hash(stripWhitespace(data[:]))
   # print stripWhitespace(data[:])
   print 'MD5 stripAll: '+hash(stripAll(data[:]))
   # print stripAll(data[:])
   print 'MD5 getAttributes: '+hash(getAttributes(data[:]))
   # print getAttributes(data[:])

if __name__=="__main__": 
   if len(sys.argv) > 1: uri = sys.argv[1]
   else: uri = 'http://www.cnn.com/'
   # doHash(uri)
   print earlHash(uri)